@@ -58,7 +58,9 @@ class PermissionRequiredMixin(object):
 
     If a `get_object()` method is defined either manually or by including
     another mixin (for example ``SingleObjectMixin``) or ``self.object`` is
-    defiend then the permission will be tested against that specific instance.
+    defined then the permission will be tested against that specific instance,
+    alternatively you can specify `get_permission_object()` method if ``self.object`` 
+    or `get_object()` does not return the object against you want to test permission 
 
     .. note:
        Testing of a permission against a specific object instance requires an
@@ -117,6 +119,11 @@ class PermissionRequiredMixin(object):
          mixing would first check for global perms, if none found, then it will
          proceed to check object level permissions.
 
+    ``PermissionRequiredMixin.permission_object``
+         *Default*: ``None``, object against which test the permission; if None fallback
+         to ``self.get_permission_object()`` which return ``self.get_object()`` 
+         or ``self.object`` by default.
+
     """
     ### default class view settings
     login_url = settings.LOGIN_URL
@@ -125,7 +132,7 @@ class PermissionRequiredMixin(object):
     return_403 = False
     raise_exception = False
     accept_global_perms = False
-
+    permission_object = None
     def get_required_permissions(self, request=None):
         """
         Returns list of permissions in format *<app_label>.<codename>* that
@@ -144,7 +151,13 @@ class PermissionRequiredMixin(object):
                 "'<app_label>.<permission codename>' but is set to '%s' instead"
                 % self.permission_required)
         return perms
-
+    
+    def get_permission_object(self):
+        if self.permission_object:
+            return self.permission_object
+        return (hasattr(self, 'get_object') and self.get_object()
+                or getattr(self, 'object', None))
+                
     def check_permissions(self, request):
         """
         Checks if *request.user* has all permissions returned by
@@ -152,8 +165,7 @@ class PermissionRequiredMixin(object):
 
         :param request: Original request.
         """
-        obj = (hasattr(self, 'get_object') and self.get_object()
-            or getattr(self, 'object', None))
+        obj = self.get_permission_object()
 
 
         forbidden = get_403_or_None(request,
