@@ -6,20 +6,20 @@ from __future__ import unicode_literals
 from django.contrib.auth.models import Group
 from django.contrib.auth.models import Permission
 from django.contrib.contenttypes.models import ContentType
-from django.db import models
 from django.db.models import Count, Q
+from django.apps import apps
 from django.shortcuts import _get_queryset
 from itertools import groupby
 
-from guardian.compat import get_user_model
 from guardian.compat import basestring
+from guardian.compat import get_user_model
 from guardian.core import ObjectPermissionChecker
 from guardian.exceptions import MixedContentTypeError
 from guardian.exceptions import WrongAppError
 from guardian.utils import get_anonymous_user
+from guardian.utils import get_group_obj_perms_model
 from guardian.utils import get_identity
 from guardian.utils import get_user_obj_perms_model
-from guardian.utils import get_group_obj_perms_model
 import warnings
 
 def assign_perm(perm, user_or_group, obj=None):
@@ -152,7 +152,7 @@ def get_perms_for_model(cls):
     """
     if isinstance(cls, basestring):
         app_label, model_name = cls.split('.')
-        model = models.get_model(app_label, model_name)
+        model = apps.get_model(app_label, model_name)
     else:
         model = cls
     ctype = ContentType.objects.get_for_model(model)
@@ -283,12 +283,16 @@ def get_groups_with_perms(obj, attach_perms=False):
                 groups[group] = sorted(get_perms(group, obj))
         return groups
 
+
 def get_objects_for_user(user, perms, klass=None, use_groups=True, any_perm=False,
-        with_superuser=True):
+        with_superuser=True, accept_global_perms=True):
     """
     Returns queryset of objects for which a given ``user`` has *all*
     permissions present at ``perms``.
 
+    If ``perms`` is an empty list, then it returns objects for which
+    a given ``user`` has *any* object permission.
+
     :param user: ``User`` or ``AnonymousUser`` instance for which objects would
       be returned.
     :param perms: single permission string, or sequence of permission strings
@@ -301,10 +305,17 @@ def get_objects_for_user(user, perms, klass=None, use_groups=True, any_perm=Fals
       this parameter would be computed based on given ``params``.
     :param use_groups: if ``False``, wouldn't check user's groups object
       permissions. Default is ``True``.
-    :param any_perm: if True, any of permission in sequence is accepted
-    :param with_superuser: if ``True`` returns the entire queryset if not it will
-    only return objects the user has explicit permissions.
-    
+    :param any_perm: if True, any of permission in sequence is accepted. Default is ``False``.
+    :param with_superuser: if ``True`` and if ``user.is_superuser`` is set,
+      returns the entire queryset. Otherwise will only return objects the user
+      has explicit permissions. This must be ``True`` for the accept_global_perms
+      parameter to have any affect. Default is ``True``.
+    :param accept_global_perms: if ``True`` takes global permissions into account.
+      Object based permissions are taken into account if more than one permission is handed in in perms and at least
+      one of these perms is not globally set. If any_perm is set to false then the intersection of matching object
+      is returned. Note, that if with_superuser is False, accept_global_perms will be ignored, which means that only
+      object permissions will be checked! Default is ``True``.
+
     :raises MixedContentTypeError: when computed content type for ``perms``
       and/or ``klass`` clashes.
     :raises WrongAppError: if cannot compute app label for given ``perms``/
@@ -323,6 +334,7 @@ def get_objects_for_user(user, perms, klass=None, use_groups=True, any_perm=Fals
         >>> get_objects_for_user(joe, 'auth.change_group')
         [<Group some group>]
 
+
     The permission string can also be an iterable. Continuing with the previous example:
 
         >>> get_objects_for_user(joe, ['auth.change_group', 'auth.delete_group'])
@@ -333,6 +345,48 @@ def get_objects_for_user(user, perms, klass=None, use_groups=True, any_perm=Fals
         >>> get_objects_for_user(joe, ['auth.change_group', 'auth.delete_group'])
         [<Group some group>]
 
+    Take global permissions into account:
+
+        >>> jack = User.objects.get(username='jack')
+        >>> assign_perm('auth.change_group', jack) # this will set a global permission
+        >>> get_objects_for_user(jack, 'auth.change_group')
+        [<Group some group>]
+        >>> group2 = Group.objects.create('other group')
+        >>> assign_perm('auth.delete_group', jack, group2)
+        >>> get_objects_for_user(jack, ['auth.change_group', 'auth.delete_group']) # this retrieves intersection
+        [<Group other group>]
+        >>> get_objects_for_user(jack, ['auth.change_group', 'auth.delete_group'], any_perm) # this retrieves union
+        [<Group some group>, <Group other group>]
+
+    If accept_global_perms is set to ``True``, then all assigned global
+    permissions will also be taken into account.
+
+    - Scenario 1: a user has view permissions generally defined on the model
+      'books' but no object based permission on a single book instance:
+
+        - If accept_global_perms is ``True``: List of all books will be
+          returned.
+        - If accept_global_perms is ``False``: list will be empty.
+
+    - Scenario 2: a user has view permissions generally defined on the model
+      'books' and also has an object based permission to view book 'Whatever':
+
+        - If accept_global_perms is ``True``: List of all books will be
+          returned.
+        - If accept_global_perms is ``False``: list will only contain book
+          'Whatever'.
+
+    - Scenario 3: a user only has object based permission on book 'Whatever':
+
+        - If accept_global_perms is ``True``: List will only contain book
+          'Whatever'.
+        - If accept_global_perms is ``False``: List will only contain book
+          'Whatever'.
+
+    - Scenario 4: a user does not have any permission:
+
+        - If accept_global_perms is ``True``: Empty list.
+        - If accept_global_perms is ``False``: Empty list.
     """
     if isinstance(perms, basestring):
         perms = [perms]
@@ -362,13 +416,13 @@ def get_objects_for_user(user, perms, klass=None, use_groups=True, any_perm=Fals
                 ctype = new_ctype
 
     # Compute queryset and ctype if still missing
-    if ctype is None and klass is None:
-        raise WrongAppError("Cannot determine content type")
-    elif ctype is None and klass is not None:
+    if ctype is None and klass is not None:
         queryset = _get_queryset(klass)
         ctype = ContentType.objects.get_for_model(queryset.model)
     elif ctype is not None and klass is None:
         queryset = _get_queryset(ctype.model_class())
+    elif klass is None:
+        raise WrongAppError("Cannot determine content type")
     else:
         queryset = _get_queryset(klass)
         if ctype.model_class() != queryset.model:
@@ -389,12 +443,40 @@ def get_objects_for_user(user, perms, klass=None, use_groups=True, any_perm=Fals
     if user.is_anonymous():
         user = get_anonymous_user()
 
+    global_perms = set()
+    has_global_perms = False
+    # a superuser has by default assigned global perms for any
+    if accept_global_perms and with_superuser:
+        for code in codenames:
+            if user.has_perm(ctype.app_label + '.' + code):
+                global_perms.add(code)
+        for code in global_perms:
+            codenames.remove(code)
+        ## prerequisite: there must be elements in global_perms otherwise just follow the procedure for
+        # object based permissions only AND
+        # 1. codenames is empty, which means that permissions are ONLY set globally, therefore return the full queryset.
+        # OR
+        # 2. any_perm is True, then the global permission beats the object based permission anyway,
+        # therefore return full queryset
+        if len(global_perms) > 0 and (len(codenames) == 0 or any_perm):
+            return queryset
+        # if we have global perms and still some object based perms differing from global perms and any_perm is set
+        # to false, then we have to flag that global perms exist in order to merge object based permissions by user
+        # and by group correctly. Scenario: global perm change_xx and object based perm delete_xx on object A for user,
+        # and object based permission delete_xx  on object B for group, to which user is assigned.
+        # get_objects_for_user(user, [change_xx, delete_xx], use_groups=True, any_perm=False, accept_global_perms=True)
+        # must retrieve object A and B.
+        elif len(global_perms) > 0 and (len(codenames) > 0):
+            has_global_perms = True
+
+
     # Now we should extract list of pk values for which we would filter queryset
     user_model = get_user_obj_perms_model(queryset.model)
     user_obj_perms_queryset = (user_model.objects
         .filter(user=user)
-        .filter(permission__content_type=ctype)
-        .filter(permission__codename__in=codenames))
+        .filter(permission__content_type=ctype))
+    if len(codenames):
+        user_obj_perms_queryset = user_obj_perms_queryset.filter(permission__codename__in=codenames)
     if user_model.objects.is_generic():
         fields = ['object_pk', 'permission__codename']
     else:
@@ -404,15 +486,18 @@ def get_objects_for_user(user, perms, klass=None, use_groups=True, any_perm=Fals
         group_model = get_group_obj_perms_model(queryset.model)
         group_filters = {
             'permission__content_type': ctype,
-            'permission__codename__in': codenames,
             'group__%s' % get_user_model().groups.field.related_query_name(): user,
         }
+        if len(codenames):
+            group_filters.update({
+                'permission__codename__in': codenames,
+            })
         groups_obj_perms_queryset = group_model.objects.filter(**group_filters)
         if group_model.objects.is_generic():
             fields = ['object_pk', 'permission__codename']
         else:
             fields = ['content_object__pk', 'permission__codename']
-        if not any_perm:
+        if not any_perm and len(codenames) and not has_global_perms:
             user_obj_perms = user_obj_perms_queryset.values_list(*fields)
             groups_obj_perms = groups_obj_perms_queryset.values_list(*fields)
             data = list(user_obj_perms) + list(groups_obj_perms)
@@ -432,17 +517,18 @@ def get_objects_for_user(user, perms, klass=None, use_groups=True, any_perm=Fals
 
     values = user_obj_perms_queryset.values_list(fields[0], flat=True)
     if user_model.objects.is_generic():
-        values = [int(v) for v in values]
+        values = list(values)
     objects = queryset.filter(pk__in=values)
     if use_groups:
         values = groups_obj_perms_queryset.values_list(fields[0], flat=True)
         if group_model.objects.is_generic():
-            values = [int(v) for v in values]
+            values = list(values)
         objects |= queryset.filter(pk__in=values)
 
     return objects
 
-def get_objects_for_group(group, perms, klass=None, any_perm=False):
+
+def get_objects_for_group(group, perms, klass=None, any_perm=False, accept_global_perms=True):
     """
     Returns queryset of objects for which a given ``group`` has *all*
     permissions present at ``perms``.
@@ -457,6 +543,9 @@ def get_objects_for_group(group, perms, klass=None, any_perm=False):
     :param klass: may be a Model, Manager or QuerySet object. If not given
       this parameter would be computed based on given ``params``.
     :param any_perm: if True, any of permission in sequence is accepted
+    :param accept_global_perms: if ``True`` takes global permissions into account.
+      If any_perm is set to false then the intersection of matching objects based on global and object based permissions
+      is returned. Default is ``True``.
 
     :raises MixedContentTypeError: when computed content type for ``perms``
       and/or ``klass`` clashes.
@@ -487,6 +576,14 @@ def get_objects_for_group(group, perms, klass=None, any_perm=False):
         >>> get_objects_for_group(group, ['tasker.add_task', 'tasker.delete_task'])
         [<Task some task>]
 
+    Global permissions assigned to the group are also taken into account. Continuing with previous example:
+        >>> task_other = Task.objects.create('other task')
+        >>> assign_perm('tasker.change_task', group)
+        >>> get_objects_for_group(group, ['tasker.change_task'])
+        [<Task some task>, <Task other task>]
+        >>> get_objects_for_group(group, ['tasker.change_task'], accept_global_perms=False)
+        [<Task some task>]
+
     """
     if isinstance(perms, basestring):
         perms = [perms]
@@ -533,6 +630,17 @@ def get_objects_for_group(group, perms, klass=None, any_perm=False):
     # match which means: ctype.model_class() == queryset.model
     # we should also have ``codenames`` list
 
+    global_perms = set()
+    if accept_global_perms:
+        global_perm_set = group.permissions.values_list('codename', flat=True)
+        for code in codenames:
+            if code in global_perm_set:
+                global_perms.add(code)
+        for code in global_perms:
+            codenames.remove(code)
+        if len(global_perms) > 0 and (len(codenames) == 0 or any_perm):
+            return queryset
+
     # Now we should extract list of pk values for which we would filter queryset
     group_model = get_group_obj_perms_model(queryset.model)
     groups_obj_perms_queryset = (group_model.objects
@@ -553,6 +661,5 @@ def get_objects_for_group(group, perms, klass=None, any_perm=False):
         obj_codenames = set((e[1] for e in group))
         if any_perm or codenames.issubset(obj_codenames):
             pk_list.append(pk)
-
     objects = queryset.filter(pk__in=pk_list)
     return objects
